/* 
This class provides all the functionality needed to interface with the html/css components.

Notes
	- This class should not know anything about NB

Orientation
	0 === Content on Left, 1 === Top, 2 === Right, 3 === Bottom 
*/

var PullView = function() {
	//Props
	this.picker = new ColorPicker();
	this._orientation = 0;
	this._rgb = [0, 193, 222];
	this._pointArrowOut = false;

	//Picker
	this.picker.jq.attr('id', 'picker');
	$('#pickerWrap').append(this.picker.jq);
	this.picker.setHexBorder(0.06, '#585858');
	this.picker.layout({
		wheelWidth: 0.9,
		wheelThickness: 0.15,
	});
	$('#pickerWrap').hide();
	
	//Other
	this.setStrings();
	this.setRgb(this._rgb , false);
	$('#settings').css({
		backgroundColor: ColorPicker.rgbToString(this.alternateColorFor(this._rgb)),
	});
	$('#contentImg').load(this.layoutImg.bind(this));
	$('#contentImgWrap').hide();

	this.layoutAutoSnap();
};

PullView.prototype = {

	//Props
	setTitle: function(title) {
		$('#titleInput').val(title);
		$('#tabTitle').text(title);
		this.layoutTitle();
		this.layoutTitleInput();
	},

	setStrings: function() {
		$('#tabPosTitle').text(strings.tabPosition);
		$('#colorTitle').text(strings.color);
		$('#contentInput').attr('placeholder', strings.content);
		$('#autoSnapText').text(strings.autoSnap);
		$('#titleInput').attr('placeholder', strings.editTitle);
	},

	setContent: function(src, text) {
		if(src) {
			this.setImg(src);
		} else {
			this.removeImg();
			$('#contentInput').val(text);
		}
	},

	setArrowPointOut: function(val) {
		this._pointArrowOut = val;
		this.layoutArrow();
	},

	btnToOrientation: function(element) {
		for(var i = 0; i < 4; i++) {
			if($('#tabPos' + i)[0] === element) {
				return i;
			}
		}
		return -1;
	},

	hideContextViews: function() {
		/* Give the user the appearance that focus has left */
		$('#pickerWrap').hide();
		this.setImgSelected(false);
	},

	//Colors
	togglePicker: function() {
		var p = $('#pickerWrap');
		if(p.is(':visible')) {
			p.hide();
		} else {
			p.show();
		}
	},

	rgb: function() {
		return this._rgb;
	},

	setRgb: function(rgb, omitPicker) {
		/* 
		Input
			rgb: [r, g, b] - 0 - 255
			omitPicker: Bool - optional, defaults to false. true if we shouldn't set the color pickers rgb.
		*/
		this._rgb = rgb;
		
		//Backgrounds
		$('#inner').css({
			backgroundColor: ColorPicker.rgbToString(this.alternateColorFor(rgb)),
		});
		if(!omitPicker) {
			this.picker.setRgb(rgb);
		}

		//Text
		var isDark = ColorPicker.isDarkText(rgb)
		var textColor = isDark ? 'black' : 'white';
		$('#tabTitle').css({
			color: textColor,
		});
	
		//Arrow
		if(isDark) {
			$('#arrow').hide();
			$('#arrow2').show();
		} else {
			$('#arrow').show();
			$('#arrow2').hide();
		}


		//Backgrounds
		var c = ColorPicker.rgbToString(rgb);
		$('#tab, #colorBox, .tabPos').css({
			backgroundColor: c,
		});

		//Tab Pos
		var border = isDark ? '#474747' : 'white';
		$('.tabPos *').css({
			backgroundColor: border,
		});
		$('.tabPos, #colorBox').css({
			border: '2px solid ' + border,
		});
	},

	alternateColorFor: function(rgb) {
		var hsl = ColorPicker.rgbToHsl(rgb);
		if(hsl[2] < 0.7) {
			hsl[2] += 0.2;
		} else {
			hsl[2] -= 0.2;
		}
		return ColorPicker.hslToRgb(hsl);
	},

	//Image
	img: function() {
		var src = $('#contentImg').attr('src');
		if(src) {
			return src;
		}
		return '';
	},

	setImg: function(imgSrc) {
		if(imgSrc) {
			$('#contentImgWrap').show();
			$('#contentImg').attr('src', imgSrc);
		} else {
			this.removeImg();
		}
	},

	removeImg: function() {
		$('#contentImg').attr('src', '');
		$('#contentImgWrap').hide();
	},

	isImgSelected: function() {
		return $('#contentImg').hasClass('contentImgSelected');
	},

	setImgSelected: function(selected) {
		if(selected) {
			$('#contentImg').addClass('contentImgSelected');
			$('#contentImg').removeClass('contentImgUnselected');
		} else {
			$('#contentImg').removeClass('contentImgSelected');
			$('#contentImg').addClass('contentImgUnselected');
		}
	},

	//Auto Snap Toggle
	isAutoSnap: function() {
		return $('#autoSnapImg').is(':visible');
	},

	setAutoSnap: function(snap) {
		if(snap) {
			$('#autoSnapImg').show();
		} else {
			$('#autoSnapImg').hide();
		}
	},

	toggleAutoSnap: function() {
		this.setAutoSnap(!this.isAutoSnap());
	},

	//Orientation
	isHorz: function() {
		return this._orientation === 0 || this._orientation === 2;
	},

	orientation: function() {
		return this._orientation;
	},

	setOrientation: function(orientation) {
		this._orientation = orientation;
		var w = $('#content').width();
		var h = $('#content').height();
		if(this.isSettingsShown()) {
			w += $('#settings').width();
		}
		if(this.isHorz()) {
			w += this.tabSize();
		} else {
			h += this.tabSize();
		}
		$('#pullView').width(w).height(h);
		this._layout();
	},

	switchOrientation: function(orientation) {
		var v = $('#pullView');
		var wOld = v.width();
		var hOld = v.height();
		this.setOrientation(orientation);
		return {
			dw: v.width() - wOld,
			dh: v.height() - hOld,
		};
	},

	//Settings
	isSettingsShown: function() {
		return Number($('#settings').css('opacity')) === 1;
	},

	setIsSettingsShown: function(shown) {
		$('#settings').css({
			opacity: shown ? 1 : 0,
		});
		var w = $('#content').width();
		if(this.isHorz()) {
			w += $('#tab').width();
		}
		if(shown) {
			w += $('#settings').width();
		}
		$('#pullView').width(w);
		this._layout();
	},

	toggleSettings: function() {
		this.setIsSettingsShown(!this.isSettingsShown());
	},

	//Main Layout
	layout: function(props) {
		/*
		Input
			props {
				width: Number,
				height: Number,
				isSettingsShown: Bool,
				orientation: Number,
			} 
		*/
		$('#pullView').width(props.width).height(props.height);
		$('#settings').css({
			opacity: props.isSettingsShown ? 1 : 0,
		});
		this._orientation = props.orientation;
		this._layout();
	},

	_layout: function() {
		this.layoutInner();
		this.layoutContent();
		this.layoutSettings();
		this.layoutTab();
		this.updateCorners();
	},

	setSize: function(w, h) {
		/* Update internal view sizes to match main view dimens. This func should be called when 
			annotation is resized. */
		$("#pullView").width(w).height(h);
		this._layout();
	},

	//Content Layout
	layoutInner: function() {
		var x = this.tabSize();
		var left = 0;
		var top = 0;
		var right = 0;
		var bottom = 0;
		switch(this._orientation) {
			case 0:
				right = x;
				break;
			case 1:
				bottom = x;
				break;
			case 2:
				left = x;
				break;
			case 3:
				top = x;
				break;
		}
		$('#inner').css({
			left: left,
			top: top,
			right: right,
			bottom: bottom,
		});
	},

	layoutContent: function() {
		var wSettings = this.isSettingsShown() ? $('#settings').width() : 0;
		var wContent = $('#pullView').width() - wSettings;
		var hContent = $('#pullView').height();
		var tabSize = this.tabSize();
		if(this.isHorz()) {
			wContent -= tabSize;
		} else {
			hContent -= tabSize;
		}
		$('#content').width(wContent)
			.height(hContent);
		$('#content').css({
			left: this._orientation === 0 ? wSettings : 0,
		});
		this.layoutImg();
		this.layoutTitleInput();
	},

	layoutTitleInput: function() {
		var input = $('#titleInput');
		var meas = $('#textMeas');
		meas.text(input.val());
		var gearWidth = $('#gear').width() + 8;
		var inputRight = 0;
		if(meas.width() > $('#contentWrap').width() - 2 * gearWidth) {
			inputRight = gearWidth;
		}
		input.css({
			right: inputRight,
		});
	},

	layoutImg: function() {
		var img = $('#contentImg');
		if(img.attr('src') === '') {
			return;
		}

		//Default Dimens
		img.width('auto')
			.height('auto');
		var imgWidth = img.width();
		var imgHeight = img.height();

		//Do Dimens
		var pad = 12;
		var wrapWidth = $('#contentImgWrap').width() - pad;
		var wrapHeight = $('#contentImgWrap').height() - pad;
		if(imgWidth > wrapWidth || imgHeight > wrapHeight) {
			var imgRatio = imgWidth / imgHeight;
			var wrapRatio = wrapWidth / wrapHeight;
			if(imgRatio > wrapRatio) {
				imgWidth = wrapWidth;
				imgHeight = imgWidth / imgRatio;
			} else {
				imgHeight = wrapHeight;
				imgWidth = imgHeight * imgRatio;
			}
		}
		img.width(imgWidth)
			.height(imgHeight);
	},

	//Settings Layout
	layoutSettings: function() {
		var b = '1px solid #999';
		if(this._orientation === 0) {
			$('#settings').css({
				left: 0,
				right: 'auto',
				borderRight: b,
				borderLeft: 'none',
			});
		} else {
			$('#settings').css({
				left: 'auto',
				right: 0,
				borderLeft: b,
				borderRight: 'none',
			});
		}
	},

	layoutAutoSnap: function() {
		/* Wrap text in center */
		$('#autoSnapText').width('auto');
		var wText = $('#autoSnapText').width();
		var wSettings = $('#settings').width();
		var margX = 2; //margin between button and edge, when at full text length
		var x1 = 18; //distance between button's left border and text's left border
		$('#autoSnapText').width(wSettings - x1 - 2 * margX);
		var wrapLeft = (wSettings - (x1 + wText)) / 2;
		if(wrapLeft < margX) {
			wrapLeft = margX;
		}
		$('#autoSnap').css({
			left: wrapLeft,
		});
	},

	//Tab Layout
	tabSize: function() {
		return 30;
	},

	layoutTab: function(orientation, width, height) {
		//Size
		var width = this.tabSize();
		var height = this.tabSize();
		if(this.isHorz()) {
			height = $('#pullView').height();
		} else {
			width = $('#content').width();
		}

		//Pos
		var left = 'auto';
		var right = 'auto';
		var top = 'auto';
		var bottom = 'auto';
		switch(this._orientation) {
			case 0:
				right = 0;
				top = 0;
				break;
			case 1:
				left = 0;
				bottom = 0;
				break;
			case 2:
				left = 0;
				top = 0;
				break;
			case 3:
				left = 0;
				top = 0;
				break;
		}

		//Write
		$('#tab').css({
			left: left,
			right: right,
			top: top,
			bottom: bottom,
			width: width,
			height: height,
		});
		this.layoutArrow();
		this.layoutTitle();
	},

	layoutTitle: function() {
		/* Layout title in center of tab */
		var tab = $('#tab');
		var title = $('#tabTitle');
		var pad1 = 4;
		var pad2 = this.isHorz() ? 30 : 24; //represents min distance between title and edge to account for arrow.
		
		//Width
		var tabLength = this.isHorz() ? tab.height() : tab.width();
		title.width('auto');
		var max = tabLength - pad1 - pad2;
		if(title.width() > max) {
			title.width(max);
		}

		//Center
		var cx = tab.width() / 2;
		var cy = tab.height() / 2;
		if(title.width() > tabLength - 2 * pad2) {
			if(this.isHorz()) {
				cy = pad2 + title.width() / 2;
			} else {
				cx = tab.width() - pad2 - title.width() / 2;
			}
		}
		title.css({
			left: cx - title.width() / 2,
			top: cy - title.height() / 2,
		});

		//Rotation
		var rotation = 0;
		if(this._orientation === 0) {
			rotation = 90;
		} else if(this._orientation === 2) {
			rotation = -90;
		}
		$('#tabTitle').css({
			transform: 'rotate(' + rotation + 'deg)',
		});
	},

	layoutArrow: function() {
		var rotate = 0;
		switch(this._orientation) {
			case 0:
				rotate = this._pointArrowOut ? 0 : 180;
				break;
			case 1:
				rotate = this._pointArrowOut ? 90: 270;
				break;
			case 2:
				rotate = this._pointArrowOut ? 180 : 0;
				break;
			case 3:
				rotate = this._pointArrowOut ? 270 : 90;
				break;
		}
		$('#arrow, #arrow2').css({
			transform: 'rotate(' + rotate + 'deg)',
		});
	},

	//Other Layout
	updateCorners: function() {
		/* Update Corner Radii for orientation */
		var rad1 = 6;
		var rad2 = 3;
		var isSettings = this.isSettingsShown();
		switch(this._orientation) {
			case 0:
				this.setCornersFor($('#tab'), 0, rad1, rad1, 0);
				this.setCornersFor($('#inner'), rad2, 0, 0, rad2);
				break;
			case 1:
				this.setCornersFor($('#tab'), 0, 0, rad1, rad1);
				this.setCornersFor($('#inner'), rad2, rad2, isSettings ? rad2 : 0, 0);
				break;
			case 2:
				this.setCornersFor($('#tab'), rad1, 0, 0, rad1);
				this.setCornersFor($('#inner'), 0, rad2, rad2, 0);
				break;
			case 3:
				this.setCornersFor($('#tab'), rad1, rad1, 0, 0);
				this.setCornersFor($('#inner'), 0, isSettings ? rad2 : 0, rad2, rad2);
				break;
		}
	},

	setCornersFor: function(jq, topLeft, topRight, bottomRight, bottomLeft) {
		var br = topLeft + 'px ' + topRight + 'px ' + bottomRight + 'px ' + bottomLeft + 'px';
		jq.css({'border-radius': br});
	},

};
